@@ -189,7 +189,7 @@ use Data::Dumper;
 use base 'Exporter';
 use v5.14;
 
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 
 our @EXPORT = qw{
 	one_row
@@ -907,21 +907,7 @@ sub make_object_delete {
 		$delete = <<DEL;
 		sub delete {
 			my \$self = \$_[0];
-			if(\@_ > 1) {
-				my (\$where, \@bind);
-				my \$cond = \$_[1];
-				if(not CORE::ref(\$cond)) {
-					\$cond = {(selectKeys)[0] => \$_[1]};
-				}
-				(\$where, \@bind) = SQL::Abstract->new->where(\$cond);
-				return DBIx::Struct::connect->run(sub {
-					\$_->do(qq{delete from $table \$where}, undef, \@bind)
-					or DBIx::Struct::error_message {
-						result  => 'SQLERR',
-						message => 'error '.\$_->errstr.' updating table $table'
-					}
-				});
-			} else {
+			if(Scalar::Util::blessed \$self) {
 				DBIx::Struct::connect->run(
 					sub {
 						\$_->do(qq{delete from $table where $pk_where}, undef, $pk_row_data)
@@ -930,8 +916,34 @@ sub make_object_delete {
 							message => 'error '.\$_->errstr.' updating table $table'
 						}
 					});
+				return \$self;
 			}
-			\$self;
+			my \$where = '';
+			my \@bind;
+			my \$cond = \$_[1] if \@_ > 1;
+			if(not CORE::ref(\$cond)) {
+				\$cond = {};
+				my \@keys = selectKeys();
+				for(my \$i = 1; \$i < \@_; ++\$i) {
+					DBIx::Struct::error_message {
+						result  => 'SQLERR',
+						message => "Too many keys to delete for $table"
+					} if not defined \$keys[\$i-1];
+					\$cond->{\$keys[\$i-1]} = \$_[\$i];
+				}
+			}
+			my \@rpar = ();
+			if(\$cond) {
+				(\$where, \@bind) = SQL::Abstract->new->where(\$cond);
+				\@rpar = (undef, \@bind);
+			}
+			return DBIx::Struct::connect->run(sub {
+				\$_->do(qq{delete from $table \$where}, \@rpar)
+				or DBIx::Struct::error_message {
+					result  => 'SQLERR',
+					message => 'error '.\$_->errstr.' updating table $table'
+				}
+			});
 		}
 DEL
 	} else {
